@@ -18,13 +18,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <GeoIP.h>
-#include <GeoIPCity.h>
-#include <GeoIP_internal.h>
+#include "GeoIP.h"
+#include "GeoIPCity.h"
+#include "GeoIP_internal.h"
 void geoiplookup(GeoIP* gi,char *hostname,int i);
 
 void usage() {
-	fprintf(stderr,"Usage: geoiplookup [-d custom_dir] [-f custom_file] [-v] <ipaddress|hostname>\n");
+	fprintf(stderr,"Usage: geoiplookup [-h] [-d custom_dir] [-f custom_file] [-v] <ipaddress|hostname>\n");
 }
 
 int main (int argc, char *argv[]) {
@@ -44,6 +44,10 @@ int main (int argc, char *argv[]) {
 	while (i < argc) {
 		if (strcmp(argv[i],"-v") == 0) {
 			version_flag = 1;
+		} else if (strcmp(argv[i], "-h") == 0 
+				|| strcmp(argv[i], "-?") == 0) {
+			usage();
+			exit(0);
 		} else if (strcmp(argv[i],"-f") == 0) {
 			if ((i+1) < argc){
 				i++;
@@ -90,7 +94,11 @@ int main (int argc, char *argv[]) {
 			if (GeoIP_db_avail(i)) {
 				gi = GeoIP_open_type(i, GEOIP_STANDARD);
 				if (NULL == gi) {
-					printf("%s not available, skipping...\n", GeoIPDBDescription[i]);
+                                        /* Ignore these errors. It's possible
+                                         * to use the same database name for
+                                         * different databases.
+                                         */
+                                        ;
 				} else {
 					if (version_flag == 1) {
 						db_info = GeoIP_database_info(gi);
@@ -116,13 +124,9 @@ geoiplookup(GeoIP * gi, char *hostname, int i)
 {
 	const char     *country_code;
 	const char     *country_name;
-	const char     *domain_name;
         const char     *asnum_name;        
-	int             netspeed;
 	int             country_id;
-	GeoIPRegion    *region;
 	GeoIPRecord    *gir;
-	const char     *org;
 
 	geoipv6_t       ipnum;
 	ipnum = _GeoIP_lookupaddress_v6(hostname);
@@ -147,7 +151,7 @@ geoiplookup(GeoIP * gi, char *hostname, int i)
 
 
 
-		if (GEOIP_LOCATIONA_EDITION_V6 == i || GEOIP_ASNUM_EDITION_V6 == i || GEOIP_USERTYPE_EDITION_V6 == i || GEOIP_REGISTRAR_EDITION_V6 == i || GEOIP_DOMAIN_EDITION_V6 == i || GEOIP_ORG_EDITION_V6 == i || GEOIP_ISP_EDITION_V6 == i ) {
+		if (GEOIP_LOCATIONA_EDITION_V6 == i || GEOIP_ASNUM_EDITION_V6 == i || GEOIP_USERTYPE_EDITION_V6 == i || GEOIP_REGISTRAR_EDITION_V6 == i || GEOIP_DOMAIN_EDITION_V6 == i || GEOIP_ORG_EDITION_V6 == i || GEOIP_ISP_EDITION_V6 == i || GEOIP_NETSPEED_EDITION_REV1_V6 == i ) {
 			asnum_name = GeoIP_name_by_ipnum_v6(gi, ipnum);
 			if (asnum_name == NULL) {
 				printf("%s: IP Address not found\n", GeoIPDBDescription[i]);
@@ -159,7 +163,7 @@ geoiplookup(GeoIP * gi, char *hostname, int i)
 		}
 	
         	else if (GEOIP_CITY_EDITION_REV0_V6 == i) {
-		        gir = GeoIP_record_by_name_v6(gi, hostname);
+		        gir = GeoIP_record_by_ipnum_v6(gi, ipnum);
       		        if (NULL == gir) {
 			        printf("%s: IP Address not found\n", GeoIPDBDescription[i]);
 		        }
@@ -169,7 +173,7 @@ geoiplookup(GeoIP * gi, char *hostname, int i)
 		        }
 	        }
 	        else if (GEOIP_CITY_EDITION_REV1_V6 == i) {
-		        gir = GeoIP_record_by_name_v6(gi, hostname);
+		        gir = GeoIP_record_by_ipnum_v6(gi, ipnum);
 		        if (NULL == gir) {
 			        printf("%s: IP Address not found\n", GeoIPDBDescription[i]);
 		        }
@@ -203,6 +207,7 @@ geoiplookup(GeoIP * gi, char *hostname, int i)
 		}
 		else {
 			printf("%s: %s, %s\n", GeoIPDBDescription[i], region->country_code, region->region);
+			GeoIPRegion_delete(region);
 		}
 	}
 	else if (GEOIP_CITY_EDITION_REV0 == i) {
