@@ -1,4 +1,3 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
 /* geoipupdate.c
  *
  * Copyright (C) 2006 MaxMind LLC
@@ -24,7 +23,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef __linux__
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 #include <ctype.h>
@@ -36,17 +35,27 @@
 
 const char *GeoIPConfFile = "GeoIP.conf";
 
+static void * realloc_or_die( void * ptr, size_t size ){
+    void * new = realloc( ptr, size );
+    if ( !new ) {
+        free(ptr);
+        fprintf(stderr, "Out of memory\n");
+        exit(1);
+    }
+    return new;
+}
+
 void usage() {
   fprintf(stderr,"Usage: geoipupdate [-hv] [-f license_file] [-d custom directory]\n");
 }
 
 void my_printf(char * str) {
-	printf(str);
+	printf("%s", str);
 }
 
 void print_status (int err, char * license_file) {
 	if (err == GEOIP_NO_NEW_UPDATES) {
-		fprintf(stderr,"GeoIP Database up to date\n");
+		fprintf(stdout,"GeoIP Database up to date\n");
 	} else if (err == GEOIP_LICENSE_KEY_INVALID_ERR) {
 		fprintf(stderr,"Invalid License Key in %s - Please visit http://www.maxmind.com/app/products for a subscription\n",license_file);
 	} else if (err == GEOIP_USER_ID_INVALID_ERR){
@@ -56,7 +65,7 @@ void print_status (int err, char * license_file) {
 	} else if (err < 0) {
 		fprintf(stderr,"Received Error %d (%s) when attempting to update GeoIP Database\n",err, GeoIP_get_error_message(err));
 	} else {
-		fprintf(stderr,"Updated database\n");
+		fprintf(stdout,"Updated database\n");
 	}
 }
 
@@ -97,6 +106,7 @@ int main (int argc, char *argv[]) {
 			exit(0);
 		case 'v':
 			verbose = 1;
+                        break;
 		case 'f':
 			license_file = optarg;
 			break;
@@ -140,7 +150,10 @@ int main (int argc, char *argv[]) {
 		c = fgetc(license_fh);
 		if (line_index >= n) {
 			n += 20;
-			lineptr = realloc(lineptr, n);
+			lineptr = realloc_or_die(lineptr, n);
+		}
+		if ( c == 13 ) {
+		  continue;
 		}
 		if (c == 10 || c == EOF) {
 			lineptr[line_index++] = '\0';
@@ -148,7 +161,7 @@ int main (int argc, char *argv[]) {
 			if (lineptr[0] == '#')
 				continue;
 			/* get the product ids from the config file */
-			a_product_id_str = strstr(lineptr, PRODUCT_ID_TOKEN);//search for a product id token in the line
+			a_product_id_str = strstr((char *)lineptr, PRODUCT_ID_TOKEN);//search for a product id token in the line
 			if (a_product_id_str != NULL) {
 				a_ptr = a_product_id_str;
 				/* set pos at the end of product id token */
@@ -172,7 +185,7 @@ int main (int argc, char *argv[]) {
 							 * alloc length of the string then realloc the string and
 							 * increase the alloc length by 20 */
 							the_product_id_stral[num_product_ids] = the_product_id_stral[num_product_ids] + 20;
-							the_product_id_str[num_product_ids] = (char *) realloc(the_product_id_str[num_product_ids],the_product_id_stral[num_product_ids]+4);
+							the_product_id_str[num_product_ids] = (char *) realloc_or_die(the_product_id_str[num_product_ids],the_product_id_stral[num_product_ids]+4);
 						}
 						/* read the product id from the line in the config file */
 						the_product_id_str[num_product_ids][the_product_id_strl[num_product_ids]] = a_ptr[0];
@@ -186,16 +199,16 @@ int main (int argc, char *argv[]) {
 					/* new product id add, realloc the arrays */
 					num_product_ids = num_product_ids + 1;
 					/* array of string */
-					the_product_id_str = (char **) realloc(the_product_id_str,(num_product_ids+1) * sizeof(char*));
+					the_product_id_str = (char **) realloc_or_die(the_product_id_str,(num_product_ids+1) * sizeof(char*));
 					/* array of string lengths */
-					the_product_id_strl = (int *) realloc(the_product_id_strl,(num_product_ids+1) * sizeof(char*));
+					the_product_id_strl = (int *) realloc_or_die(the_product_id_strl,(num_product_ids+1) * sizeof(char*));
 					/* array of string alloc lengths */
-					the_product_id_stral = (int *) realloc(the_product_id_stral,(num_product_ids+1) * sizeof(char*));
+					the_product_id_stral = (int *) realloc_or_die(the_product_id_stral,(num_product_ids+1) * sizeof(char*));
 				}
 			}
 
 			/* get the user id from the config file */
-			a_user_id_str = strstr(lineptr, USER_ID_TOKEN); /* search for a user id token in the line */
+			a_user_id_str = strstr((char *)lineptr, USER_ID_TOKEN); /* search for a user id token in the line */
 			if (a_user_id_str != NULL) {
 				a_ptr = a_user_id_str;
 				/* set the position at the end of user id token */
@@ -216,12 +229,12 @@ int main (int argc, char *argv[]) {
 						 * the alloc length of user id string then
 						 * add 20 to the alloc length and realloc the user id string */
 						the_user_id_stral += 20;
-						the_user_id_str = realloc(the_user_id_str,the_user_id_stral);
+						the_user_id_str = realloc_or_die(the_user_id_str,the_user_id_stral);
 					}
 				}
 				the_user_id_str[the_user_id_strl] = 0; /* add NUL char */
 			}
-			a_license_key_str = strstr(lineptr, LICENSE_KEY_TOKEN);
+			a_license_key_str = strstr((char *)lineptr, LICENSE_KEY_TOKEN);
 			if (a_license_key_str != NULL) {
 				a_ptr = a_license_key_str;
 				a_ptr += strlen(LICENSE_KEY_TOKEN) + 1;
@@ -279,5 +292,5 @@ int main (int argc, char *argv[]) {
 	if (client_ipaddr) {
 		free(client_ipaddr);
 	}
-	exit(0);
+	exit(err);
 }
